# how to run a single test:
# $ python -m unittest tests.test_demo.Main.test_amici1

from lino.utils.pythontest import TestCase


class Main(TestCase):

    demo_projects_root = 'lino_amici/projects'

    def test_amici1(self):
        self.do_test_demo_project('amici1')



# -*- coding: UTF-8 -*-
# Copyright 2014-2021 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""This is the main module of Lino Amici.

.. autosummary::
   :toctree:

   lib


"""

from .setup_info import SETUP_INFO

__version__ = SETUP_INFO['version']

intersphinx_urls = dict(docs="https://lino-framework.gitlab.io/amici/")
srcref_url = 'https://gitlab.com/lino-framework/amici/blob/master/%s'
doc_trees = ['docs']



# -*- coding: UTF-8 -*-
# Copyright 2017 Luc Saffre
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""Shows Lino Amici as a family management tool.

.. autosummary::
   :toctree:

   settings

"""




from lino.api import dd, _
from lino_xl.lib.households.models import *

class HouseholdDetail(dd.DetailLayout):
    main = "general more"

    general = dd.Panel("""
    address_box
    bottom_box
    """, label=_("General"))

    address_box = "overview:40 data_box:30 lists.MembersByPartner:30"

    data_box = """
    type
    prefix
    name
    language id
    """

    more = dd.Panel("""
    uploads.UploadsByController:30 comments.CommentsByRFC:50
    """, label=_("More"))

    bottom_box = "households.MembersByHousehold"



#!/usr/bin/env python
if __name__ == "__main__":
    import sys
    import os
    os.environ['DJANGO_SETTINGS_MODULE'] = \
        'lino_amici.projects.amici1.settings.demo'
    from lino.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)



"""
Examples how to run these tests::

  $ python setup.py test
  $ python setup.py test -s tests.DocsTests
  $ python setup.py test -s tests.DocsTests.test_debts
  $ python setup.py test -s tests.DocsTests.test_docs
"""

from lino_amici import SETUP_INFO

from lino.utils.pythontest import TestCase

class PackagesTests(TestCase):

    def test_packages(self):
        self.run_packages_test(SETUP_INFO['packages'])



# -*- coding: UTF-8 -*-
# Copyright 2014-2020 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""
Base Django settings for Lino Amici applications.

"""

from lino.projects.std.settings import *
from lino.api.ad import _
from lino_amici import SETUP_INFO

class Site(Site):

    verbose_name = "Lino Amici"
    description = SETUP_INFO['description']
    version = SETUP_INFO['version']
    url = SETUP_INFO['url']

    demo_fixtures = ['std', 'demo', 'demo2', 'checkdata']
                     # 'linotickets',
                     # 'tractickets', 'luc']

    # project_model = 'tickets.Project'
    # project_model = 'deploy.Milestone'
    textfield_format = 'html'
    custom_layouts_module = 'lino_amici.lib.amici.custom_layouts'
    user_types_module = 'lino_amici.lib.amici.user_types'
    workflows_module = 'lino_amici.lib.amici.workflows'
    obj2text_template = "**{0}**"

    # default_build_method = 'appyodt'
    default_build_method = 'weasy2pdf'
    default_ui = 'lino_react.react'

    # migration_class = 'lino_amici.lib.amici.migrate.Migrator'

    auto_configure_logger_names = "atelier django lino lino_xl lino_amici"

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        # yield 'lino.modlib.extjs'
        # yield 'lino.modlib.bootstrap3'
        # yield 'lino.modlib.gfks'
        # yield 'lino.modlib.system'
        # yield 'lino.modlib.users'
        yield 'lino.modlib.help'
        yield 'lino_amici.lib.users'
        yield 'lino_amici.lib.contacts'
        # yield 'lino_xl.lib.online.users'
        yield 'lino.modlib.checkdata'
        yield 'lino_amici.lib.cal'
        yield 'lino_xl.lib.calview'
        yield 'lino_xl.lib.sepa'
        # yield 'lino_xl.lib.extensible'
        yield 'lino_xl.lib.addresses'
        yield 'lino_xl.lib.phones'
        yield 'lino_amici.lib.households'
        yield 'lino_xl.lib.humanlinks'
        # yield 'lino_xl.lib.cv'
        yield 'lino_xl.lib.courses'
        # yield 'lino_noi.lib.products'

        yield 'lino_xl.lib.topics'
        yield 'lino_xl.lib.tickets'
        # yield 'lino_xl.lib.votes'
        # yield 'lino_xl.lib.skills'
        # yield 'lino_xl.lib.deploy'
        # yield 'lino_noi.lib.working'
        yield 'lino_xl.lib.lists'
        yield 'lino_xl.lib.blogs'

        # yield 'lino.modlib.changes'
        # yield 'lino.modlib.notify'
        yield 'lino.modlib.uploads'
        # yield 'lino_xl.lib.outbox'
        yield 'lino_xl.lib.excerpts'
        yield 'lino.modlib.export_excel'
        yield 'lino.modlib.tinymce'
        # yield 'lino.modlib.smtpd'
        yield 'lino.modlib.weasyprint'
        yield 'lino_xl.lib.appypod'
        # yield 'lino.modlib.wkhtmltopdf'
        yield 'lino.modlib.comments'
        yield 'lino.modlib.dashboard'

        # yield 'lino.modlib.awesomeuploader'

        # yield 'lino_noi.lib.noi'
        yield 'lino_amici.lib.amici'
        yield 'lino_xl.lib.google'
        # yield 'lino_xl.lib.inbox'
        # yield 'lino_xl.lib.mailbox'
        # yield 'lino_xl.lib.meetings'

    def get_plugin_configs(self):
        for i in super(Site, self).get_plugin_configs():
            yield i
        # yield ('addresses', 'partner_model', 'contacts.Person')
        yield 'cal', 'partner_model', 'contacts.Person'
        yield 'cal', 'with_demo_absences', False
        yield 'cal', 'calendar_fieldnames', 'room__calendar'
        yield 'cal', 'summary_length', 600
        yield 'contacts', 'with_roles_history', True

    # def setup_plugins(self):
    #     super(Site, self).setup_plugins()
    #     # self.plugins.comments.configure(
    #     #     commentable_model='tickets.Ticket')
    #     # self.plugins.skills.configure(
    #     #     demander_model='contacts.Person')
    #     self.plugins.addresses.configure(
    #         partner_model='contacts.Person')
    #     self.plugins.cal.configure(
    #         partner_model='contacts.Person')
    #     self.plugins.contacts.configure(
    #         with_roles_history=True)

        # self.plugins.skills.configure(
        #     demander_model='tickets.Ticket')
        # self.plugins.tickets.configure(
        #     site_model='cal.Room',
        #     milestone_model='courses.Course')

    def setup_quicklinks(self, ut, tb):
        super(Site, self).setup_quicklinks(ut, tb)
        tb.add_action(self.models.contacts.Persons)
        tb.add_action(self.models.households.Households)
        tb.add_action(self.models.contacts.Companies)

        # a = self.models.users.MySettings.default_action
        # tb.add_instance_action(
        #     user, action=a, label=_("My settings"))

        # tb.add_action(
        #     self.models.blogs.MyEntries.insert_action,
        #     label=_("New blog entry"))


# the following line should not be active in a checked-in version
#~ DATABASES['default']['NAME'] = ':memory:'

USE_TZ = True
# TIME_ZONE = 'Europe/Brussels'
# TIME_ZONE = 'Europe/Tallinn'
TIME_ZONE = 'UTC'



from atelier.invlib import setup_from_tasks
ns = setup_from_tasks(
    globals(), "lino_amici",
    languages="en de fr et".split(),
    tolerate_sphinx_warnings=False,
    blogref_url='https://luc.lino-framework.org',
    revision_control_system='git',
    locale_dir='lino_amici/lib/amici/locale',
    cleanable_files=[
        'docs/api/lino_amici.*'],
    demo_projects=[
        'lino_amici.projects.amici1']
)



# -*- coding: UTF-8 -*-
# Copyright 2017-2020 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)

from lino.api import dd, rt, _
from lino.utils import join_words
from lino.mixins import Hierarchical

from lino_xl.lib.contacts.models import *
from lino.modlib.comments.mixins import Commentable
# from lino_xl.lib.cal.mixins import BirthdayGenerator

Partner.define_action(show_problems=dd.ShowSlaveTable(
    'checkdata.MessagesByOwner', icon_name='bell', combo_group="checkdata"))


class PartnerDetail(PartnerDetail):

    main = """
    overview #address_box:60 contact_box:30
    bottom_box
    """

    # address_box = dd.Panel("""
    # name_box
    # country #region city zip_code:10
    # addr1
    # #street_prefix street:25 street_no street_box
    # #addr2
    # """)  # , label=_("Address"))

    contact_box = dd.Panel("""
    name id
    url
    phone
    gsm #fax
    """, label=_("Contact"))

    bottom_box = """
    remarks lists.MembersByPartner
    """



Partners.detail_layout = 'contacts.PartnerDetail'


class Person(Person, Commentable):  # , BirthdayGenerator):

    class Meta(Person.Meta):
        abstract = dd.is_abstract_model(__name__, 'Person')

    def __str__(self):
        words = []
        words.append(self.first_name)
        words.append(self.last_name)
        return join_words(*words)

    def get_address_parent(self):
        p = super(Person, self).get_address_parent()
        if p is None:
            Member = rt.models.households.Member
            try:
                return Member.objects.get(
                    person=self, primary=True).household
            except Member.DoesNotExist:
                pass

    # def get_overview_elems(self, ar):
    #     elems = super(Person, self).get_overview_elems(ar)
    #     elems += AddressOwner.get_overview_elems(self, ar)
    #     elems += ContactDetailsOwner.get_overview_elems(self, ar)
    #     return elems

    @classmethod
    def setup_parameters(cls, fields):
        fields.setdefault(
            'company', dd.ForeignKey(
                'contacts.Company', blank=True, null=True))
        fields.setdefault(
            'function', dd.ForeignKey(
                'contacts.RoleType', blank=True, null=True))
        fields.setdefault(
            'topic', dd.ForeignKey(
                'topics.Topic', blank=True, null=True))
        super(Person, cls).setup_parameters(fields)

    @classmethod
    def get_simple_parameters(cls):
        lst = list(super(Person, cls).get_simple_parameters())
        lst.append('company')
        lst.append('function')
        lst.append('topic')
        return lst

    @classmethod
    def add_param_filter(cls, qs, lookup_prefix='', company=None,
                         function=None,
                         topic=None, **kwargs):
        qs = super(Person, cls).add_param_filter(qs, **kwargs)
        if company:
            fkw = dict()
            wanted = company.whole_clan()
            fkw[lookup_prefix + 'rolesbyperson__company__in'] = wanted
            qs = qs.filter(**fkw)

        if function:
            fkw = dict()
            fkw[lookup_prefix + 'rolesbyperson__type'] = function
            qs = qs.filter(**fkw)

        if topic:
            fkw = dict()
            wanted = topic.whole_clan()
            fkw[lookup_prefix + 'interest_set__topic__in'] = wanted
        return qs


    # @classmethod
    # def get_request_queryset(cls, ar):
    #     qs = super(Person, cls).get_request_queryset(ar)
    #     pv = ar.param_values
    #     if pv.skill:
    #     return qs

# We use the `overview` field only in detail forms, and we
# don't want it to have a label "Description":
dd.update_field(Person, 'overview', verbose_name=None)

class Company(Company, Hierarchical, Commentable):

    class Meta(Company.Meta):
        abstract = dd.is_abstract_model(__name__, 'Company')


    # def get_overview_elems(self, ar):
    #     elems = super(Company, self).get_overview_elems(ar)
    #     # elems += AddressOwner.get_overview_elems(self, ar)
    #     elems += ContactDetailsOwner.get_overview_elems(self, ar)
    #     return elems


class PersonDetail(PersonDetail):

    main = "general #contact #career family more"

    general = dd.Panel("""
    navigation_panel:20 general_box:60
    """, label=_("General"))

    general_box = """
    overview contact_box #phones.ContactDetailsByPartner
    contacts.RolesByPerson:30 lists.MembersByPartner:30 cal.EntriesByGuest:30
    """

    contact_box = dd.Panel("""
    last_name first_name:15
    gender #title:10 language:10
    birth_date age:10 id:6
    """)  #, label=_("Contact"))

    # contact = dd.Panel("""
    # address_box
    # remarks
    # """, label=_("Contact"))

    # career = dd.Panel("""
    # cv.StudiesByPerson
    # # cv.TrainingsByPerson
    # cv.ExperiencesByPerson:40
    # """, label=_("Career"))

    family = dd.Panel("""
    humanlinks.LinksByHuman:50 households.MembersByPerson:30
    households.SiblingsByPerson
    """, label=_("Family"))

    more = dd.Panel("""
    more_left comments.CommentsByRFC:30
    """, label=_("More"))

    more_left = """
    remarks
    checkdata.MessagesByOwner:20 sepa.AccountsByPartner:30
    uploads.UploadsByController:20
    """

class CompaniesByCompany(Companies):
    label = _("Child organisations")
    master_key = 'parent'
    column_names = 'name_column email id *'


class CompanyDetail(CompanyDetail):
    main = "general contact more"

    general = dd.Panel("""
    overview:30 data_box:30
    contacts.RolesByCompany:30 sepa.AccountsByPartner:30
    """, label=_("General"))

    data_box = """
    prefix
    name
    type
    language id
    # parent
    """
    contact = dd.Panel("""
    # address_box
    CompaniesByCompany lists.MembersByPartner:30
    remarks checkdata.MessagesByOwner
    """, label=_("Contact"))

    more = dd.Panel("""
    uploads.UploadsByController:20 comments.CommentsByRFC:50
    """, label=_("More"))



# @dd.receiver(dd.post_analyze)
# def my_details(sender, **kw):
#     contacts = sender.models.contacts
#     contacts.Companies.set_detail_layout(contacts.CompanyDetail())

# Companies.set_detail_layout(CompanyDetail())
# Persons.set_detail_layout(PersonDetail())
Person.column_names = 'last_name first_name gsm email city *'
Persons.params_layout = 'observed_event start_date end_date company function topic'
Persons.insert_layout = """
first_name last_name
gender country language
email
gsm
"""
Persons.react_responsive = False
#Persons.react_big_search = True
Persons.column_names = 'last_name first_name *'



# -*- coding: UTF-8 -*-
# Copyright 2013-2016 Luc Saffre
# License: GNU Affero General Public License v3 (see file COPYING for details)

"""Database models for :mod:`lino_welfare.modlib.users`.

"""

from lino.modlib.users.models import *



from .ui import *



# Copyright 2014-2016 Luc Saffre
# License: GNU Affero General Public License v3 (see file COPYING for details)

"""
Lino Amici extension of :mod:`lino.modlib.users`.


"""

from lino.modlib.users import Plugin


# class Plugin(Plugin):
    
#     extends_models = ['User']




# -*- coding: UTF-8 -*-
# Copyright 2017-2022 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)

"""Defines the user types for Lino Amici.

This is used as the :attr:`user_types_module
<lino.core.site.Site.user_types_module>` for Amici sites.

"""

from lino.core.roles import UserRole, SiteUser, SiteAdmin
from lino_xl.lib.excerpts.roles import ExcerptsUser, ExcerptsStaff
from lino_xl.lib.contacts.roles import ContactsUser, ContactsStaff
from lino_xl.lib.courses.roles import CoursesUser
from lino_xl.lib.blogs.roles import BlogsReader
from lino_xl.lib.sepa.roles import SepaStaff
from lino.modlib.office.roles import OfficeStaff, OfficeUser
from lino.modlib.checkdata.roles import CheckdataUser
from lino.modlib.search.roles import SiteSearcher
from lino_xl.lib.cal.roles import GuestOperator

from lino.modlib.comments.roles import CommentsUser, CommentsStaff, PrivateCommentsReader
from lino_xl.lib.tickets.roles import Triager, TicketsStaff, Reporter, Searcher, TicketsReader
from lino_xl.lib.votes.roles import VotesStaff, VotesUser
from lino_xl.lib.working.roles import Worker

#from lino_xl.lib.cal.roles import CalendarReader

from lino.modlib.users.choicelists import UserTypes
from django.utils.translation import gettext_lazy as _


class EndUser(SiteUser, OfficeUser, GuestOperator, BlogsReader, CheckdataUser,
              TicketsReader, Searcher, Reporter):
    pass


class Collector(EndUser, ExcerptsUser, ContactsUser, SiteSearcher):
    pass


class ProjectManager(Collector, VotesUser, Triager, CommentsUser, Worker):
    pass


class Staff(ProjectManager, CoursesUser, ExcerptsStaff, CommentsStaff,
            VotesStaff, TicketsReader, Searcher, Reporter, TicketsStaff, SepaStaff):
    pass


class SiteAdmin(Staff, SiteAdmin, OfficeStaff, ContactsStaff, PrivateCommentsReader):
    pass

# class Anonymous(CommentsReader, CalendarReader):
class Anonymous(UserRole):
    pass

UserTypes.clear()
add = UserTypes.add_item
add('000', _("Anonymous"),        Anonymous, 'anonymous',
    readonly=True, authenticated=False)
add('100', _("User"),             EndUser, 'user')
add('200', _("Collector"),        Collector, 'collector')
# add('300', _("Project manager"),  ProjectManager, 'manager')
# add('800', _("Staff"),            Staff, 'staff')
add('900', _("Administrator"),    SiteAdmin, 'admin')



from datetime import timedelta
from lino.api.shell import *
from django.db.models.functions import Right
qs = contacts.Person.objects.exclude(birth_date='')
qs = qs.annotate(bday=Right("birth_date",5)).exclude(bday="00-00").order_by('bday')

recent = qs.filter(bday__gt=str(dd.today()-timedelta(days=3))[-5:]).filter(bday__lt=str(dd.today())[-5:])
coming = qs.filter(bday__lt=str(dd.today()+timedelta(days=3))[-5:]).filter(bday__gt=str(dd.today())[-5:])
today = qs.filter(bday=str(dd.today())[-5:])

print("Birthdays today:", ", ".join(["{} {}".format(obj.birth_date, obj) for obj in today]))
print("Recent birthdays:", ", ".join(["{} {}".format(obj.birth_date, obj) for obj in recent]))
print("Upcoming birthdays:", ", ".join(["{} {}".format(obj.birth_date, obj) for obj in coming]))



# -*- coding: UTF-8 -*-
# Copyright 2014-2018 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""
The main plugin for Lino Amici.

.. autosummary::
   :toctree:

    user_types
    workflows

"""

from lino.api.ad import Plugin


class Plugin(Plugin):
    pass




# -*- coding: UTF-8 -*-
# Copyright 2017-2023 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)

# $ python setup.py test -s tests.test_packages

SETUP_INFO = dict(
    name='lino-amici',
    version='23.11.0',
    install_requires=['lino-xl', 'vobject'],

    # tests_require=['pytest', 'mock'],
    test_suite='tests',
    description=("A Lino for managing family contacts"),
    long_description="""\

**Lino Amici** is a Lino Django application for managing family contacts.

- Source code: https://gitlab.com/lino-framework/amici

- Documentation: https://lino-framework.gitlab.io/amici/

- Changelog: https://lino-framework.gitlab.io/amici/changes.html

- This is an integral part of the Lino framework, which is documented
  at https://www.lino-framework.org

- For introductions, commercial information and hosting solutions
  see https://www.saffre-rumma.net

- This is a sustainably free open-source project. Your contributions are
  welcome.  See https://community.lino-framework.org for details.


""",
    author='Rumma & Ko Ltd',
    author_email='info@lino-framework.org',
    url="https://gitlab.com/lino-framework/amici",
    license_files=['COPYING'],
    classifiers="""\
Programming Language :: Python
Programming Language :: Python :: 3
Development Status :: 4 - Beta
Environment :: Web Environment
Framework :: Django
Intended Audience :: Developers
Intended Audience :: System Administrators
Intended Audience :: Information Technology
Intended Audience :: Customer Service
License :: OSI Approved :: GNU Affero General Public License v3
Operating System :: OS Independent
Topic :: Software Development :: Bug Tracking
Topic :: Communications :: Email :: Address Book
Topic :: Office/Business :: Groupware
""".splitlines())

SETUP_INFO.update(packages=[str(n) for n in """
lino_amici
lino_amici.lib
lino_amici.lib.amici
lino_amici.lib.cal
lino_amici.lib.contacts
lino_amici.lib.contacts.fixtures
lino_amici.lib.households
lino_amici.lib.households.fixtures
lino_amici.projects
lino_amici.projects.amici1
lino_amici.projects.amici1.settings
lino_amici.projects.amici1.settings.fixtures
lino_amici.projects.amici1.tests
lino_amici.lib.users
lino_amici.lib.users.fixtures
""".splitlines() if n])

SETUP_INFO.update(message_extractors={
    'lino_amici': [
        ('**/cache/**',          'ignore', None),
        ('**.py',                'python', None),
        ('**.js',                'javascript', None),
        ('**/config/**.html', 'jinja2', None),
    ],
})

SETUP_INFO.update(include_package_data=True, zip_safe=False)
# SETUP_INFO.update(package_data=dict())


# def add_package_data(package, *patterns):
#     l = SETUP_INFO['package_data'].setdefault(package, [])
#     l.extend(patterns)
#     return l

# l = add_package_data('lino_noi.lib.noi')
# for lng in 'de fr'.split():
#     l.append('locale/%s/LC_MESSAGES/*.mo' % lng)



# import os ; print "20161219 %s (pid:%s)" % (__name__, os.getpid())

import datetime
from lino.core.auth.utils import activate_social_auth_testing
from ..settings import *


class Site(Site):
    is_demo_site = True
    the_demo_date = datetime.date(2019, 12, 16)
    languages = "en de fr"
    # is_demo_site = False

    # default_ui = 'lino.modlib.extjs'
    # default_ui = 'lino.modlib.bootstrap3'
    # default_user = 'anonymous'

    def setup_features(self):
        super().setup_features()
        self.enable_feature('third_party_authentication')


activate_social_auth_testing(globals(), google=False, github=False, wikimedia=False)

SITE = Site(globals())
# print "20161219 b"
DEBUG = True

# the following line should not be active in a checked-in version
# DATABASES['default']['NAME'] = ':memory:'
